/*
 * ImportExportFactory.java
 *
 * Created on 20 de Outubro de 2005, 18:20
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package demoapp;

import common.ApplicationException;
import dynclassloader.FileClassLoader;
import dynclassloader.JarClassLoader;
import dynclassloader.URLClassLoader;
import importexport.ImportExport;
import java.io.FileInputStream;
import java.util.Properties;

/**
 *
 * @author Paulo Sousa
 */
public class ImportExportFactory {
    
    /** Creates a new instance of ImportExportFactory */
    public ImportExportFactory() {
    }
    
    public ImportExport CreateProvider() throws ApplicationException
    {
        try
        {
            // create and load default properties
            Properties defaultProps = new Properties(); 
            FileInputStream in = new FileInputStream("c:\\temp\\demoapp.properties");
            defaultProps.load(in); 
            in.close(); 

            // ler valores de configurao
            String classname = defaultProps.getProperty("ImpExpProviderClass");
            if (classname=="")
                throw new ApplicationException("falta configurar provider");
            String jarname = defaultProps.getProperty("ImpExpProviderSource");
            if (jarname=="")
                throw new ApplicationException("falta configurar provider");

            // carregar a classe dinamicamente 
            //Class cl = Class.forName(classname);
            
            // Create the class loader 
            ClassLoader loader = null;
            if (jarname.toLowerCase().endsWith("jar"))
            {
                loader = new JarClassLoader (jarname);
            }
            else if (jarname.toLowerCase().endsWith("\\"))
            {
                loader = new FileClassLoader(jarname);
            }
            else
            {
                loader = new URLClassLoader(jarname);
            }
            

            /* Load the class from the jar file and resolve it. */
            Class cl = loader.loadClass(classname);

            //cria uma nova instancia usando construtor por defeito
            Object obj = cl.newInstance();

            if (obj instanceof ImportExport)
                    return (ImportExport)obj;
            else
                throw new ApplicationException("classe no implementa interface adequado");
            }
        catch (java.io.FileNotFoundException ex)
        {
            throw new ApplicationException("ficheiro no encontrado", ex);
        }
        catch (java.io.IOException ex)
        {
            throw new ApplicationException("erro de IO", ex);
        }
        catch (IllegalAccessException ex)
        {
            throw new ApplicationException("no foi possivel criar objecto provider", ex);
        }
        catch (InstantiationException ex)
        {
            throw new ApplicationException("no foi possivel criar objecto provider", ex);
        }
        catch (ClassNotFoundException ex)
        {
            throw new ApplicationException("classe  provider no encontrado", ex);
        }
    }
}
